@@ -1,13 +1,14 @@
 use std::convert::TryInto;
 use std::env;
-use std::io::{self, Read, Seek, SeekFrom, Write};
+use std::fmt::Write as _;
+use std::io::{self, Read, Seek, Write};
 use std::path::{Path, PathBuf};
 use std::process;
 
 use anyhow::{anyhow, Result};
+use arboard::Clipboard;
 use atty::Stream;
 use clap::ArgMatches;
-use clipboard::{ClipboardContext, ClipboardProvider};
 use daemonize::Daemonize;
 use inquire::Confirm;
 use nix::unistd::{fork, ForkResult};
@@ -16,6 +17,7 @@ use secrecy::{ExposeSecret, Secret};
 use crate::kbs2::agent;
 use crate::kbs2::backend::{self, Backend};
 use crate::kbs2::config::{self, Pinentry};
+use crate::kbs2::generator::Generator;
 use crate::kbs2::input::Input;
 use crate::kbs2::record::{
     self, EnvironmentFields, LoginFields, Record, RecordBody, UnstructuredFields,
@@ -46,7 +48,7 @@ pub fn init(matches: &ArgMatches, config_dir: &Path) -> Result<()> {
 
     #[allow(clippy::unwrap_used)]
     let password = if !*matches.get_one::<bool>("insecure-not-wrapped").unwrap() {
-        Some(util::get_password(None, &Pinentry::default())?)
+        Some(util::get_password(None, Pinentry::default())?)
     } else {
         None
     };
@@ -206,13 +208,13 @@ pub fn list(matches: &ArgMatches, config: &config::Config) -> Result<()> {
             display.push_str(&label);
 
             if details {
-                display.push_str(&format!(" {} {}", record.body, record.timestamp));
+                write!(display, " {} {}", record.body, record.timestamp)?;
             }
         } else {
             display.push_str(&label);
         }
 
-        println!("{}", display);
+        println!("{display}");
     }
 
     Ok(())
@@ -305,37 +307,16 @@ pub fn pass(matches: &ArgMatches, config: &config::Config) -> Result<()> {
         unsafe {
             match fork() {
                 Ok(ForkResult::Child) => {
-                    // NOTE(ww): More dumbness: cfg! gets expanded into a boolean literal,
-                    // so it can't be used to conditionally compile code that only exists on
-                    // one platform.
-                    #[cfg(target_os = "linux")]
-                    {
-                        match session.config.commands.pass.x11_clipboard {
-                            // NOTE(ww): Why, might you ask, is clip_primary its own function?
-                            // It's because the clipboard crate has a bad abstraction:
-                            // ClipboardContext is the top-level type, but it's aliased to
-                            // X11Clipboard<Clipboard>. That means we can't produce it on a match.
-                            // The other option would be to create a ClipboardProvider trait object,
-                            // but it doesn't implement Sized. So we have to do things the dumb
-                            // way here. Alternatively, I could just be missing something obvious.
-                            config::X11Clipboard::Primary => clip_primary(password, &session)?,
-                            config::X11Clipboard::Clipboard => clip(password, &session)?,
-                        };
-                    }
-
-                    #[cfg(target_os = "macos")]
-                    {
-                        clip(password, &session)?;
-                    }
+                    clip(password, &session)?;
                 }
                 Err(_) => return Err(anyhow!("clipboard fork failed")),
                 _ => {}
             }
         }
     } else if atty::isnt(Stream::Stdout) {
-        print!("{}", password);
+        print!("{password}");
     } else {
-        println!("{}", password);
+        println!("{password}");
     }
 
     if let Some(post_hook) = &session.config.commands.pass.post_hook {
@@ -351,44 +332,13 @@ fn clip(password: String, session: &Session) -> Result<()> {
     let clipboard_duration = session.config.commands.pass.clipboard_duration;
     let clear_after = session.config.commands.pass.clear_after;
 
-    let mut ctx: ClipboardContext =
-        ClipboardProvider::new().map_err(|_| anyhow!("unable to grab the clipboard"))?;
-    ctx.set_contents(password)
-        .map_err(|_| anyhow!("unable to store to the clipboard"))?;
+    let mut clipboard = Clipboard::new()?;
+    clipboard.set_text(&password)?;
 
     std::thread::sleep(std::time::Duration::from_secs(clipboard_duration));
 
     if clear_after {
-        ctx.set_contents("".to_owned())
-            .map_err(|_| anyhow!("unable to clear the clipboard"))?;
-
-        if let Some(clear_hook) = &session.config.commands.pass.clear_hook {
-            log::debug!("clear-hook: {}", clear_hook);
-            session.config.call_hook(clear_hook, &[])?;
-        }
-    }
-
-    Ok(())
-}
-
-#[doc(hidden)]
-#[cfg(target_os = "linux")]
-fn clip_primary(password: String, session: &Session) -> Result<()> {
-    use clipboard::x11_clipboard::{Primary, X11ClipboardContext};
-
-    let clipboard_duration = session.config.commands.pass.clipboard_duration;
-    let clear_after = session.config.commands.pass.clear_after;
-
-    let mut ctx: X11ClipboardContext<Primary> =
-        ClipboardProvider::new().map_err(|_| anyhow!("unable to grab the clipboard"))?;
-    ctx.set_contents(password)
-        .map_err(|_| anyhow!("unable to store to the clipboard"))?;
-
-    std::thread::sleep(std::time::Duration::from_secs(clipboard_duration));
-
-    if clear_after {
-        ctx.set_contents("".to_owned())
-            .map_err(|_| anyhow!("unable to clear the clipboard"))?;
+        clipboard.clear()?;
 
         if let Some(clear_hook) = &session.config.commands.pass.clear_hook {
             log::debug!("clear-hook: {}", clear_hook);
@@ -459,14 +409,14 @@ pub fn edit(matches: &ArgMatches, config: &config::Config) -> Result<()> {
     if !process::Command::new(&editor)
         .args(&editor_args)
         .arg(file.path())
-        .output()
-        .map_or(false, |o| o.status.success())
+        .status()
+        .map_or(false, |o| o.success())
     {
         return Err(anyhow!("failed to run the editor"));
     }
 
     // Rewind, pull the changed contents, deserialize back into a record.
-    file.seek(SeekFrom::Start(0))?;
+    file.rewind()?;
     let mut record_contents = vec![];
     file.read_to_end(&mut record_contents)?;
 
@@ -527,10 +477,7 @@ pub fn rewrap(matches: &ArgMatches, config: &config::Config) -> Result<()> {
         }
 
         std::fs::copy(&config.keyfile, &keyfile_backup)?;
-        println!(
-            "Backup of the OLD wrapped keyfile saved to: {:?}",
-            keyfile_backup
-        );
+        println!("Backup of the OLD wrapped keyfile saved to: {keyfile_backup:?}");
     }
 
     let old = util::get_password(Some("OLD master password: "), &config.pinentry)?;
@@ -575,10 +522,7 @@ pub fn rekey(matches: &ArgMatches, config: &config::Config) -> Result<()> {
         }
 
         std::fs::copy(&config.keyfile, &keyfile_backup)?;
-        println!(
-            "Backup of the OLD wrapped keyfile saved to: {:?}",
-            keyfile_backup
-        );
+        println!("Backup of the OLD wrapped keyfile saved to: {keyfile_backup:?}");
 
         // Next, the config itself.
         let config_backup: PathBuf =
@@ -593,7 +537,7 @@ pub fn rekey(matches: &ArgMatches, config: &config::Config) -> Result<()> {
             Path::new(&config.config_dir).join(config::CONFIG_BASENAME),
             &config_backup,
         )?;
-        println!("Backup of the OLD config saved to: {:?}", config_backup);
+        println!("Backup of the OLD config saved to: {config_backup:?}");
 
         // Finally, every record in the store.
         let store_backup: PathBuf = format!("{}.old", &config.store).into();
