@@ -20,9 +20,6 @@ use crate::kbs2::util;
 /// directory.
 pub static CONFIG_BASENAME: &str = "config.toml";
 
-/// A deprecated alternative default config basename.
-pub static LEGACY_CONFIG_BASENAME: &str = "kbs2.conf";
-
 /// The default generate age key is placed in this file, relative to
 /// the configuration directory.
 pub static DEFAULT_KEY_BASENAME: &str = "key";
@@ -147,15 +144,10 @@ impl Config {
 
     /// Given the `name` of a configured generator, return that generator
     /// if it exists.
-    pub fn generator(&self, name: &str) -> Option<&dyn Generator> {
-        for generator_config in self.generators.iter() {
-            let generator = generator_config.as_dyn();
-            if generator.name() == name {
-                return Some(generator);
-            }
-        }
-
-        None
+    pub fn generator(&self, name: &str) -> Option<&GeneratorConfig> {
+        self.generators
+            .iter()
+            .find(|&generator_config| generator_config.name() == name)
     }
 
     /// Create a `RuntimeConfig` from this config and the given `matches`.
@@ -183,43 +175,8 @@ impl AsRef<OsStr> for Pinentry {
     }
 }
 
-/// The different types of generators known to `kbs2`.
 #[derive(Clone, Debug, Deserialize, Serialize)]
-#[serde(untagged)]
-pub enum GeneratorConfig {
-    Command(ExternalGeneratorConfig),
-    Internal(InternalGeneratorConfig),
-    InternalLegacy(LegacyInternalGeneratorConfig),
-}
-
-impl GeneratorConfig {
-    fn as_dyn(&self) -> &dyn Generator {
-        match self {
-            GeneratorConfig::Command(g) => g as &dyn Generator,
-            GeneratorConfig::Internal(g) => g as &dyn Generator,
-            GeneratorConfig::InternalLegacy(g) => g as &dyn Generator,
-        }
-    }
-}
-
-impl Default for GeneratorConfig {
-    fn default() -> Self {
-        GeneratorConfig::Internal(Default::default())
-    }
-}
-
-/// The configuration settings for an external (i.e., separate command) generator.
-#[derive(Clone, Debug, Deserialize, Serialize)]
-pub struct ExternalGeneratorConfig {
-    /// The name of the generator.
-    pub name: String,
-
-    /// The command to run to generate a secret.
-    pub command: String,
-}
-
-#[derive(Clone, Debug, Deserialize, Serialize)]
-pub struct InternalGeneratorConfig {
+pub struct GeneratorConfig {
     /// The name of the generator.
     pub name: String,
 
@@ -230,9 +187,9 @@ pub struct InternalGeneratorConfig {
     pub length: usize,
 }
 
-impl Default for InternalGeneratorConfig {
+impl Default for GeneratorConfig {
     fn default() -> Self {
-        InternalGeneratorConfig {
+        GeneratorConfig {
             name: "default".into(),
             alphabets: vec![
                 "abcdefghijklmnopqrstuvwxyz".into(),
@@ -245,23 +202,6 @@ impl Default for InternalGeneratorConfig {
     }
 }
 
-/// The configuration settings for a legacy "internal" generator.
-///
-/// This is a **legacy** generator that will be removed in an upcoming release.
-///
-/// Users should prefer `InternalGeneratorConfig`.
-#[derive(Clone, Debug, Deserialize, Serialize)]
-pub struct LegacyInternalGeneratorConfig {
-    /// The name of the generator.
-    pub name: String,
-
-    /// The alphabet to sample from when generating a secret.
-    pub alphabet: String,
-
-    /// The length of the secrets generated.
-    pub length: u32,
-}
-
 /// The per-command configuration settings known to `kbs2`.
 #[derive(Clone, Default, Debug, Deserialize, Serialize)]
 #[serde(default)]
@@ -305,8 +245,6 @@ pub struct PassConfig {
     pub clipboard_duration: u64,
     #[serde(rename = "clear-after")]
     pub clear_after: bool,
-    #[serde(rename = "x11-clipboard")]
-    pub x11_clipboard: X11Clipboard,
     #[serde(deserialize_with = "deserialize_optional_with_tilde")]
     #[serde(rename = "pre-hook")]
     pub pre_hook: Option<String>,
@@ -318,7 +256,7 @@ pub struct PassConfig {
     pub clear_hook: Option<String>,
 }
 
-#[derive(Copy, Clone, Debug, Deserialize, PartialEq, Serialize)]
+#[derive(Copy, Clone, Debug, Deserialize, PartialEq, Eq, Serialize)]
 pub enum X11Clipboard {
     Clipboard,
     Primary,
@@ -329,7 +267,6 @@ impl Default for PassConfig {
         PassConfig {
             clipboard_duration: 10,
             clear_after: true,
-            x11_clipboard: X11Clipboard::Clipboard,
             pre_hook: None,
             post_hook: None,
             clear_hook: None,
@@ -364,7 +301,7 @@ pub struct RuntimeConfig<'a> {
 }
 
 impl<'a> RuntimeConfig<'a> {
-    pub fn generator(&self) -> Result<&dyn Generator> {
+    pub fn generator(&self) -> Result<&GeneratorConfig> {
         // If the user explicitly requests a specific generator, use it.
         // Otherwise, use the default generator, which is always present.
         if let Some(generator) = self.matches.get_one::<String>("generator") {
@@ -390,8 +327,8 @@ fn deserialize_with_tilde<'de, D>(deserializer: D) -> std::result::Result<String
 where
     D: de::Deserializer<'de>,
 {
-    let unexpanded: &str = Deserialize::deserialize(deserializer)?;
-    Ok(shellexpand::tilde(unexpanded).into_owned())
+    let unexpanded: String = Deserialize::deserialize(deserializer)?;
+    Ok(shellexpand::tilde(&unexpanded).into_owned())
 }
 
 #[doc(hidden)]
@@ -402,10 +339,10 @@ fn deserialize_optional_with_tilde<'de, D>(
 where
     D: de::Deserializer<'de>,
 {
-    let unexpanded: Option<&str> = Deserialize::deserialize(deserializer)?;
+    let unexpanded: Option<String> = Deserialize::deserialize(deserializer)?;
 
     match unexpanded {
-        Some(unexpanded) => Ok(Some(shellexpand::tilde(unexpanded).into_owned())),
+        Some(unexpanded) => Ok(Some(shellexpand::tilde(&unexpanded).into_owned())),
         None => Ok(None),
     }
 }
@@ -474,7 +411,7 @@ pub fn initialize<P: AsRef<Path>>(
             post_hook: None,
             error_hook: None,
             reentrant_hooks: false,
-            generators: vec![GeneratorConfig::Internal(Default::default())],
+            generators: vec![Default::default()],
             commands: Default::default(),
         })?
     };
@@ -490,17 +427,7 @@ pub fn load<P: AsRef<Path>>(config_dir: P) -> Result<Config> {
     let config_dir = config_dir.as_ref();
     let config_path = config_dir.join(CONFIG_BASENAME);
 
-    let contents = if config_path.is_file() {
-        fs::read_to_string(config_path)?
-    } else {
-        // Try the legacy config file. This behavior will be removed in a future stable release.
-        util::warn(&format!(
-            "{} not found in config dir; trying {}",
-            CONFIG_BASENAME, LEGACY_CONFIG_BASENAME
-        ));
-        util::warn("note: this behavior will be removed in a future stable release");
-        fs::read_to_string(config_dir.join(LEGACY_CONFIG_BASENAME))?
-    };
+    let contents = fs::read_to_string(config_path)?;
 
     let mut config = Config {
         config_dir: config_dir
@@ -515,14 +442,6 @@ pub fn load<P: AsRef<Path>>(config_dir: P) -> Result<Config> {
         config.generators.push(Default::default());
     }
 
-    // Warn if the user has any old-style generators.
-    for gen in config.generators.iter() {
-        if matches!(gen, GeneratorConfig::InternalLegacy(_)) {
-            util::warn(&format!("loaded legacy generator: {}", gen.as_dyn().name()));
-            util::warn("note: this behavior will be removed in a future stable release");
-        }
-    }
-
     Ok(config)
 }
 
@@ -545,7 +464,7 @@ mod tests {
             post_hook: Some("false".into()),
             error_hook: Some("true".into()),
             reentrant_hooks: false,
-            generators: vec![GeneratorConfig::Internal(Default::default())],
+            generators: vec![Default::default()],
             commands: CommandConfigs {
                 rm: RmConfig {
                     post_hook: Some("this-command-does-not-exist".into()),
